@@ -1,9 +1,10 @@
 #!/usr/bin/env python3
 
 import warnings
-from typing import Optional, Tuple, Union
+from typing import List, Optional, Tuple, Union
 
 import numpy as np
+import numpy.testing as nptest
 import pandas as pd
 import pandas.testing as pdtest
 import scipy.sparse
@@ -31,6 +32,18 @@ def series_if_applicable(ds: Union[pd.Series, pd.DataFrame]):
     return ds
 
 
+def assert_equal_eigenvectors(eigvec1, eigvec2, tol=1e-14):
+    # Allows to also check orthogonality, but is not yet implemented
+    norms1 = np.linalg.norm(eigvec1, axis=0)
+    norms2 = np.linalg.norm(eigvec2, axis=0)
+    eigvec_test = (eigvec1.conj().T @ eigvec2) * np.reciprocal(np.outer(norms1, norms2))
+
+    actual = np.abs(np.diag(eigvec_test))  # -1 is also allowed for same direction
+    expected = np.ones(actual.shape[0])
+
+    nptest.assert_allclose(expected, actual, atol=tol, rtol=0)
+
+
 def is_df_same_index(
     df_left: pd.DataFrame,
     df_right: pd.DataFrame,
@@ -119,6 +132,43 @@ def if1dim_rowvec(vec: np.ndarray):
         return vec
 
 
+def projection_matrix_from_features(
+    features_all: pd.Index, features_select: pd.Index
+) -> scipy.sparse.csr_matrix:
+    """Compute a sparse projection matrix that maps that selects columns from a matrix.
+
+    .. math::
+        A \cdot P = A^*
+
+    If matrix :math:`A` has a set of features (column-oriented), then the projection
+    matrix :math:`P` selects the requested sub-selection of features in matrix
+    :math:`A^*` (by performing the matrix multiplication).
+
+    Parameters
+    ----------
+    features_all
+        All features in the original matrix.
+
+    features_select
+        The features to include in the final matrix after the projection.
+
+    Returns
+    -------
+    scipy.sparse.csr_matrix
+        The projection matrix.
+    """
+    project_indices = np.where(np.isin(features_all, features_select))[0]
+
+    if len(project_indices) != len(features_select):
+        raise ValueError(
+            "Not all features from 'feature_select' are contained in 'features_all'."
+        )
+
+    project_matrix = scipy.sparse.lil_matrix((len(features_all), len(features_select)))
+    project_matrix[project_indices, np.arange(len(features_select))] = 1
+    return project_matrix.tocsr()
+
+
 def sort_eigenpairs(
     eigenvalues: np.ndarray, eigenvectors: np.ndarray, ascending: bool = False
 ) -> Tuple[np.ndarray, np.ndarray]:
@@ -154,8 +204,12 @@ def sort_eigenpairs(
             f"number of eigenvectors (={eigenvectors.shape[1]})"
         )
 
-    # Sort eigenvectors according to absolute value of eigenvalue:
-    idx = np.abs(eigenvalues).argsort()
+    # Sort eigenvectors according to (complex) value of eigenvalue
+    #  -- NOTE: the ordering according to the complex value is preferred over the
+    #           ordering of absolute value because often complex conjugate eigenvalues
+    #           have the same abs. value which makes sorting typically unstable (i.e.
+    #           there can be two equivalent but different according to the complex order)
+    idx = np.argsort(eigenvalues)
 
     if not ascending:
         # creates a view on array and is most efficient way for reversing order
@@ -219,6 +273,46 @@ def diagmat_dot_mat(diag_elements: np.ndarray, matrix: np.ndarray, out=None):
     return np.multiply(matrix, diag_elements[:, np.newaxis], out=out)
 
 
+def df_type_and_indices_from(
+    indices_from: pd.DataFrame,
+    values: Union[np.ndarray, pd.DataFrame],
+    except_index: Optional[Union[pd.Index, List[str]]] = None,
+    except_columns: Optional[Union[pd.Index, List[str]]] = None,
+):
+    # import here to prevent circular imports
+    from datafold.pcfold import TSCDataFrame
+
+    if except_index is not None and except_columns is not None:
+        raise ValueError(
+            "'except_index' and 'except_columns' are both given. "
+            "Cannot copy neither index nor column from existing TSCDataFrame if both "
+            "is excluded."
+        )
+
+    # view input as array (allows for different input, which is
+    # compatible with numpy.ndarray
+    values = np.asarray(values)
+
+    if except_index is None:
+        index = indices_from.index  # type: ignore  # mypy cannot infer type here
+    else:
+        index = except_index
+
+    if except_columns is None:
+        columns = indices_from.columns
+    else:
+        columns = except_columns
+
+    if isinstance(indices_from, TSCDataFrame):
+        return TSCDataFrame(data=values, index=index, columns=columns)
+    elif isinstance(indices_from, pd.DataFrame):
+        return pd.DataFrame(data=values, index=index, columns=columns)
+    else:
+        raise TypeError(
+            f"The argument type 'type(indices_from)={type(indices_from)} is invalid."
+        )
+
+
 def is_symmetric_matrix(
     matrix: Union[np.ndarray, scipy.sparse.csr_matrix], tol: float = 0
 ) -> bool:
@@ -227,9 +321,9 @@ def is_symmetric_matrix(
     Parameters
     ----------
     matrix
-        square matrix
+       A square matrix to be checked for symmetry.
     tol
-        maximum allowed absolute deviation between corresponding elements
+       The maximum allowed absolute deviation between corresponding elements.
 
     Returns
     -------
